

//-------------------------------------------------<                   axlib v1.1                    >----------------------------------------------------
//-------------------------------------------------<             I2C        >----------------------------------------------------
//-------------------------------------------------<      2015 http://www.avrki.ru    >----------------------------------------------------

#ifndef I2C_H_
#define I2C_H_

#if !defined(MAIN_INIT_H_)
#error "You must included (#include \"main_init.h\") befor use (#include <axlib/i2c.h>)."
#endif

//-------------------------------------------------------------------------
//						  
//-------------------------------------------------------------------------

#define ACK  0		//  
#define NACK 1		//   

#ifdef SSD1306_DELAY()
	#define I2C_DELAY()	_delay_us(1);	//    
#else
	#define I2C_DELAY()	_delay_us(10);	//    
#endif

#define ONE_SCL()	{ I2C_DDR &= ~(1 << SCL); I2C_PORT |= (1 << SCL); }  //     SCL
#define NULL_SCL()	{ I2C_DDR |= (1 << SCL);  I2C_PORT &= ~(1 << SCL); } //     SCL
#define ONE_SDA()	{ I2C_DDR &= ~(1 << SDA); I2C_PORT |= (1 << SDA); }  //     SDA
#define NULL_SDA()	{ I2C_DDR |= (1 << SDA);  I2C_PORT &= ~(1 << SDA); } //     SDA
	
#define OK				0	//   
#define SCL_FAIL		1	//   SCL
#define SDA_FAIL		2	//   SDA
#define SDA_SCL_FAIL	3	//   SCL  SDA

#define WR		0   //   
#define RD		1	//   
#define NO		2	//   

//-------------------------------------------------------------------------
//							 
//-------------------------------------------------------------------------

#include <axlib/type_var.h>

//-------------------------------------------------------------------------
//							 
//-------------------------------------------------------------------------

//-------------------------------------------------------------------------
//	  .
//
//	 OK 
//
//-------------------------------------------------------------------------

BYTE i2c_stop(void)
{
	BYTE error = OK;
	
	NULL_SCL();
	I2C_DELAY();
	NULL_SDA();
	I2C_DELAY();
	
	ONE_SCL();
	I2C_DELAY();
	ONE_SDA();
	I2C_DELAY();
	
	if((I2C_PIN & (1 << SDA)) == 0) error = SDA_FAIL;
	if((I2C_PIN & (1 << SCL)) == 0) error += SCL_FAIL;
	I2C_DELAY();
	I2C_DELAY();
	I2C_DELAY();
	I2C_DELAY();
	
	return error;
}

//-------------------------------------------------------------------------
//	  .
//
//-------------------------------------------------------------------------

void i2c_start(void)
{
	NULL_SDA();
	I2C_DELAY();
	NULL_SCL();
	I2C_DELAY();
}

//-------------------------------------------------------------------------
//	  .
//
//-------------------------------------------------------------------------

void i2c_restart(void)
{
	ONE_SDA();
	I2C_DELAY();
	ONE_SCL();
	I2C_DELAY();
	
	NULL_SDA();
	I2C_DELAY();
	NULL_SCL();
	I2C_DELAY();
}

//-------------------------------------------------------------------------
//	  .
//
//-------------------------------------------------------------------------

void i2c_init(void)
{
	ONE_SDA();
	ONE_SCL();
	
	i2c_stop();
}

//-------------------------------------------------------------------------
//	    .
//
//	 :
//
//		BYTE data -     
//
//   :
//
//		    ACK , NACK .
//-------------------------------------------------------------------------

BYTE i2c_send_byte(BYTE data)
{
	BYTE i;
	BYTE ask = ACK;
	
	for(i = 0; i < 8; i++)
	{
		if((data & 0x80) == 0)
		{
			NULL_SDA();
		}
		else
		{
			ONE_SDA();
		}
		I2C_DELAY();
		ONE_SCL();
		I2C_DELAY();
		NULL_SCL();
		
		data = (data << 1);
	}
	
	ONE_SDA();
	I2C_DELAY();
	ONE_SCL();
	I2C_DELAY();
	
	if((I2C_PIN & (1 << SDA)) == (1 << SDA))
	{
		ask = NACK;
	}
	else
	{
		ask = ACK;
	}
	
	NULL_SCL();
	
	return ask;
}

//-------------------------------------------------------------------------
//	    .
//
//	  
//
//		BYTE ask -       ACK  NACK
//
//   :
//
//		 .
//-------------------------------------------------------------------------

BYTE i2c_read_byte(BYTE ask)
{
	BYTE byte = 0;
	BYTE i;
	
	ONE_SDA();
	
	for(i = 0; i < 8; i++)
	{
		byte = (byte << 1);
		ONE_SCL();
		I2C_DELAY();
		if((I2C_PIN & (1 << SDA)) == (1 << SDA)) byte |= 0x01;
		NULL_SCL();
		I2C_DELAY();
	}
	
	if(ask == ACK)
	{
		NULL_SDA();
	}
	else
	{
		ONE_SDA();
	}
	I2C_DELAY();
	ONE_SCL();
	I2C_DELAY();
	NULL_SCL();
	I2C_DELAY();
	ONE_SDA();
	
	return byte;
}

#endif /* I2C_H_ */